<?php
/**
 * JeroenVermeulen_Solarium
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this Module to
 * newer versions in the future.
 *
 * @category    JeroenVermeulen
 * @package     JeroenVermeulen_Solarium
 * @copyright   Copyright (c) 2014 Jeroen Vermeulen (http://www.jeroenvermeulen.eu)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

/**
 * Class JeroenVermeulen_Solarium_Model_Resource_CatalogSearch_Fulltext
 */
class JeroenVermeulen_Solarium_Model_Resource_CatalogSearch_Fulltext extends Mage_CatalogSearch_Model_Resource_Fulltext
{

    /**
     * This function is called when a visitor searches
     *
     * @param Mage_CatalogSearch_Model_Fulltext $object
     * @param string $queryText
     * @param Mage_CatalogSearch_Model_Query $query
     * @return JeroenVermeulen_Solarium_Model_Resource_CatalogSearch_Fulltext
     */
    public
    function prepareResult(
        $object,
        $queryText,
        $query
    ) {
        if (JeroenVermeulen_Solarium_Model_Engine::isEnabled( $query->getStoreId() )) {
            $helper            = Mage::helper('jeroenvermeulen_solarium');
            $adapter           = $this->_getWriteAdapter();
            $searchResultTable = $this->getTable( 'catalogsearch/result' );
            $catSearchHelper   = Mage::helper('catalogsearch');
            /** @var JeroenVermeulen_Solarium_Model_Engine $engine */
            $engine = Mage::getSingleton( 'jeroenvermeulen_solarium/engine' );
            if ($engine->isWorking()) {
                $searchResult = $engine->search( $query->getStoreId(), $queryText );
                $searchResult->setUserQuery( $queryText );
                Mage::register( 'solarium_search_result', $searchResult );
                if ( ! $searchResult->getResultCount() ) {
                    // Autocorrect
                    if ( $engine->getConf( 'results/autocorrect', $query->getStoreId() ) ) {
                        $searchResult->autoCorrect();
                    }
                }
                $resultProducts = $searchResult->getResultProducts();
                if ( ! $searchResult->getResultCount() ) {
                    // No results, we need to check if the index is empty.
                    if ($engine->isEmpty( $query->getStoreId() )) {
                        Mage::Log( sprintf( '%s - Warning: index is empty', __CLASS__ ), Zend_Log::WARN );
                    } else {
                        $query->setIsProcessed( 1 );
                    }
                } else {
                    $columns    = array( 'query_id', 'product_id', 'relevance' );
                    $insertRows = array();
                    $queryId    = $query->getId();
                    foreach ($resultProducts as $data) {
                        $insertRows[ ] = array( $queryId, $data[ 'product_id' ], $data[ 'relevance' ] );
                    }
                    $adapter->beginTransaction();
                    $adapter->delete( $searchResultTable, 'query_id = ' . $queryId );
                    $adapter->insertArray( $searchResultTable, $columns, $insertRows );
                    $adapter->commit();
                    $query->setIsProcessed( 1 );
                }
                // Autocorrect notification
                if ( $searchResult->didAutoCorrect() ) {
                    $catSearchHelper->addNoteMessage(
                        $helper->__( "Showing results for '%s' instead.", $searchResult->getResultQuery() ) );
                }
                // "Did you mean" suggestions
                $suggestions = $searchResult->getBetterSuggestions();
                if ( $suggestions ) {
                    $suggestHtml = '';
                    foreach ($suggestions as $searchTerm => $result_count)  {
                        $title = $helper->__('Results').': '.$result_count;
                        $href = Mage::getUrl('catalogsearch/result', array('q' => $searchTerm));
                        $suggestHtml .= sprintf('&nbsp; <a title="%s" href="%s">%s</a>', $title, $href, $searchTerm);
                    }
                    $catSearchHelper->addNoteMessage( $helper->__('Did you mean:') . $suggestHtml );
                }
                /** @deprecated The registry key 'solarium_suggest' is deprecated, it was used in 1.6.0 till 1.6.2 */
                Mage::register( 'solarium_suggest', $searchResult->getBetterSuggestions() );
            }
        }
        if (!$query->getIsProcessed()) {
            Mage::log( 'Solr disabled or something went wrong, fallback to Magento Fulltext Search', Zend_Log::WARN );
            return parent::prepareResult( $object, $queryText, $query );
        }
        return $this;
    }

    /**
     * Override to prevent table locking during cleanup of previous search results by Magento
     * - Update `catalogsearch_query`.`is_processed` in steps of 100
     * - Clean `catalogsearch_result` in steps of 100
     *
     * @return Mage_CatalogSearch_Model_Resource_Fulltext
     */
    public function resetSearchResults()
    {
        $read = $this->_getReadAdapter();
        $write = $this->_getWriteAdapter();

        $pageSize    = 100;

        $queryTable  = $this->getTable('catalogsearch/search_query');
        $querySelect = $read->select()->from( $queryTable, 'COUNT(*)' )->where( 'is_processed' );
        $queryCount  = intval( $read->fetchOne( $querySelect ) );
        $queryPages  = ceil( $queryCount / $pageSize );
        $querySql    = sprintf( 'UPDATE %s SET %s=0 WHERE %s=1 LIMIT %d',
                                $read->quoteIdentifier( $queryTable ),
                                $read->quoteIdentifier( 'is_processed' ),
                                $read->quoteIdentifier( 'is_processed' ),
                                $pageSize );
        for ( $page=0; $page < $queryPages; $page++ ) {
            try {
                // It would be better to do this using a Varien or Zend object, but they don't support LIMIT on update.
                $write->query( $querySql );
            } catch ( Exception $e ) {
                // This happens on busy sites because of deadlock
                Mage::log( sprintf('Update error during search reindex, but we can continue: %s', $e->getMessage()),
                           Zend_Log::ERR );
                sleep(1);
                $page--;
            }
        }

        $resultTable  = $this->getTable('catalogsearch/result');
        $resultSelect = $read->select()->from( $resultTable, 'COUNT(*)' );
        $resultCount  = intval( $read->fetchOne( $resultSelect ) );
        $resultPages  = ceil( $resultCount / $pageSize );
        $resultSql    = sprintf( 'DELETE FROM %s LIMIT %d',
                                 $read->quoteIdentifier( $resultTable ),
                                 $pageSize );
        for ( $page=0; $page < $resultPages; $page++ ) {
            try {
                // It would be better to do this using a Varien or Zend object, but they don't support LIMIT on delete.
                $write->query( $resultSql );
                break;
            } catch ( Exception $e ) {
                // This happens on busy sites because of deadlock
                Mage::log( sprintf('Delete error during search reindex, but we can continue: %s', $e->getMessage()),
                           Zend_Log::ERR );
                sleep(1);
                $page--;
            }
        }

        Mage::dispatchEvent('catalogsearch_reset_search_result');

        return $this;
    }

}